package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.system.service.data.sys_organization.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.base.entity.*;
import pps.core.base.enums.WindDirection;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_wind_turbine.*;
import pps.core.base.service.data.weather_bureau_data.GetWeatherBureauDataOutput;
import pps.core.base.service.data.wind_prediction_future.GetWindPredictionFutureInput;
import pps.core.base.service.data.wind_prediction_future.GetWindPredictionFutureOutput;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 风预测
 *
 * @author ZWT
 * @date 2024/08/29
 */
@XText("风预测")
@XService
public class WindPredictionFutureService {

    /**
     * 中期10天预测
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWindPredictionFutureOutput }>
     */
    @XApiPost
    @XText("中期10天预测")
    public XListResult<GetWindPredictionFutureOutput> getMidTerm10DayList(XContext context, GetWindPredictionFutureInput input) {
        return XListResult.success(getPredictedPowerOutput(context, input.getDataTime(), input.getStationId(), 10));
    }

    /**
     * 中期3天预测
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWindPredictionFutureOutput }>
     */
    @XApiPost
    @XText("中期3天预测")
    public XListResult<GetWindPredictionFutureOutput> getMidTerm3DayList(XContext context, GetWindPredictionFutureInput input) {
        return XListResult.success(getPredictedPowerOutput(context, input.getDataTime(), input.getStationId(), 3));
    }

    /**
     * 短期4小时预测
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWindPredictionFutureOutput }>
     */
    @XApiPost
    @XText("短期4小时预测")
    public XListResult<GetWindPredictionFutureOutput> getShortTerm4HourList(XContext context, GetWindPredictionFutureInput input) {
        DateTime now = DateUtil.date();
        DateTime inputTime;
        String inputStartTime = input.getDataTime();
        if (CharSequenceUtil.isBlank(inputStartTime)) {
            inputTime = DateUtil.beginOfHour(now);
        } else {
            inputTime = DateUtil.parse(now.toString(BusinessConstant.DATE_FORMAT_DAY) + " " + inputStartTime + BusinessConstant.INITIALIZATION_SECOND);
        }
        //时间出力截至时间
        DateTime day = inputTime;
        //计算时间范围,开始时间往前推1天，结束时间往后推4小时
        DateTime beginTime = DateUtil.offsetDay(day, -1);
        DateTime endTime = DateUtil.offsetHour(day, 4);
        return XListResult.success(getPowerOutput(context, beginTime, endTime, 0, input.getStationId()));
    }

    /*-----------------------------------统计分析(风能发电监控) todo 模拟数据 -----------------------------------*/

    /**
     * 场站发电量统计
     *  todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetStationPowerGenerationOutput }>
     */
    @XApiPost
    @XText("场站发电量统计")
    public XSingleResult<GetStationPowerGenerationOutput> stationPowerGeneration(XContext context, GetBaseWindTurbineViewInput input) {
        List<BaseWindTurbineEnt> turbineList = this.getWindTurbineList(context, input.getOuId());
        if (CollUtil.isEmpty(turbineList)) {
            return XSingleResult.success(new GetStationPowerGenerationOutput());
        }
        DateTime now = DateUtil.date();
        DateTime beginOfMonth = DateUtil.beginOfMonth(now);
        DateTime endOfMonth = DateUtil.endOfMonth(now);
        DateTime beginOfYear = DateUtil.beginOfYear(now);
        DateTime endOfYear = DateUtil.endOfYear(now);
        //计算季度
        DateTime beginOfSeason;
        DateTime endOfSeason;
        String year = String.valueOf(now.year());
        int month = now.monthBaseOne();
        if (month <= 3) {
            beginOfSeason = beginOfYear;
            endOfSeason = DateUtil.endOfMonth(DateUtil.offsetMonth(beginOfSeason, 2));
        } else if (month <= 6) {
            beginOfSeason = DateUtil.parse(year + "-04-01");
            endOfSeason = DateUtil.parse(year + "-06-30");
        } else if (month <= 9) {
            beginOfSeason = DateUtil.parse(year + "-07-01");
            endOfSeason = DateUtil.parse(year + "-09-30");
        } else {
            beginOfSeason = DateUtil.beginOfMonth(DateUtil.offsetMonth(endOfYear, -2));
            endOfSeason = endOfYear;
        }
        List<DateTime> dateList = CollUtil.list(false, DateUtil.beginOfDay(now), DateUtil.endOfDay(now),
                beginOfMonth, endOfMonth,
                DateUtil.offset(beginOfMonth, DateField.YEAR, -1), DateUtil.offset(endOfMonth, DateField.YEAR, -1),
                beginOfSeason, endOfSeason,
                DateUtil.offset(beginOfSeason, DateField.YEAR, -1), DateUtil.offset(endOfSeason, DateField.YEAR, -1),
                beginOfYear, endOfYear,
                DateUtil.offset(beginOfYear, DateField.YEAR, -1), DateUtil.offset(endOfYear, DateField.YEAR, -1));
        //统计
        ThirdWindPowerGenerationUpdateViewMapper viewMapper = context.getBean(ThirdWindPowerGenerationUpdateViewMapper.class);
        ThirdWindPowerGenerationUpdateView stationedPowerGeneration = viewMapper.stationPowerGeneration(turbineList.stream()
                .map(BaseWindTurbineEnt::getId)
                .collect(Collectors.toList()), dateList);
        return XSingleResult.success(GetStationPowerGenerationOutput.builder()
                .dailyGeneration(stationedPowerGeneration.getTodayPower())
                .monthGeneration(stationedPowerGeneration.getMonthPower())
                .monthGenerationRatio(BaseUtils.getCycleRatio(stationedPowerGeneration.getMonthPower(), stationedPowerGeneration.getLastMonthPower()))
                .quarterlyGeneration(stationedPowerGeneration.getSeasonPower())
                .quarterlyGenerationRatio(BaseUtils.getCycleRatio(stationedPowerGeneration.getSeasonPower(), stationedPowerGeneration.getLastSeasonPower()))
                .yearGeneration(stationedPowerGeneration.getYearPower())
                .yearGenerationRatio(BaseUtils.getCycleRatio(stationedPowerGeneration.getYearPower(), stationedPowerGeneration.getLastYearPower()))
                .build());
    }

    /**
     * 发电功率预测
     * todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerGenerationTrendOutput }>
     */
    @XApiPost
    @XText("发电功率预测")
    public XListResult<GetPowerGenerationTrendOutput> powerGenerationTrend(XContext context, GetBaseWindTurbineViewInput input) {
        DateTime now = DateUtil.date();
        //处理时间范围
        Date startTime = input.getStartTime();
        Date endTime = input.getEndTime();
        if (ObjectUtil.isNull(startTime)) {
            startTime = DateUtil.beginOfDay(now);
        } else {
            startTime = DateUtil.beginOfHour(startTime);
        }
        if (ObjectUtil.isNull(endTime)) {
            endTime = DateUtil.offsetDay(startTime, BusinessConstant.TEN);
        } else {
            endTime = DateUtil.beginOfHour(endTime);
        }
        //查电站列表
        List<BaseWindTurbineEnt> turbineList = this.getWindTurbineList(context, input.getOuId());
        if (CollUtil.isEmpty(turbineList)) {
            return XListResult.success(Collections.emptyList());
        }
        List<String> stationIds = turbineList.stream()
                .map(BaseModel::getId)
                .collect(Collectors.toList());
        //实际发电量
        ThirdWindPowerGenerationMapper generationMapper = context.getBean(ThirdWindPowerGenerationMapper.class);
        List<ThirdWindPowerGenerationEnt> generationList = generationMapper.selectList(new QueryWrapper<ThirdWindPowerGenerationEnt>()
                .select("IFNULL( SUM( actual_power ), 0 ) AS actual_power", "collect_time")
                .groupBy("collect_time")
                .lambda()
                .in(ThirdWindPowerGenerationEnt::getStationId, stationIds)
                .eq(ThirdWindPowerGenerationEnt::getSystemSource, ServiceUtil.getOilFieldCode(context))
                .between(ThirdWindPowerGenerationEnt::getCollectTime, startTime, endTime)
        );
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(generationList)) {
            powerMap = generationList.stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationEnt::getCollectTime, ThirdWindPowerGenerationEnt::getActualPower));
        } else {
            powerMap = Collections.emptyMap();
        }
        //计算中期截至时间
        Date midEnd = DateUtil.offsetDay(now, BusinessConstant.TEN);
        if (DateUtil.compare(midEnd, endTime) > 0) {
            midEnd = endTime;
        }
        Map<Date, BigDecimal> midMap = this.getPredictedMap(context, stationIds, startTime, midEnd);
        //计算短期截至时间
        Date shortEnd = DateUtil.offsetDay(now, BusinessConstant.THREE);
        if (DateUtil.compare(shortEnd, endTime) > 0) {
            shortEnd = endTime;
        }
        Map<Date, BigDecimal> shortMap = this.getPredictedMap(context, stationIds, startTime, shortEnd);
        //计算超短期截至时间
        Date ultraShortEnd = DateUtil.offsetHour(now, BusinessConstant.FOUR);
        if (DateUtil.compare(ultraShortEnd, endTime) > 0) {
            ultraShortEnd = endTime;
        }
        Map<Date, BigDecimal> ultraShortMap = this.getPredictedMap(context, stationIds, startTime, ultraShortEnd);
        //时间区间
        List<DateTime> rangeToList = DateUtil.rangeToList(startTime, endTime, DateField.MINUTE, 15);
        //封装数据
        List<GetPowerGenerationTrendOutput> outputs = new ArrayList<>(rangeToList.size());
        for (DateTime dateTime : rangeToList) {
            outputs.add(
                    GetPowerGenerationTrendOutput.builder()
                            .dataTime(dateTime)
                            .midPredictedPower(midMap.getOrDefault(dateTime, BigDecimal.ZERO))
                            .shortPredictedPower(shortMap.getOrDefault(dateTime, BigDecimal.ZERO))
                            .ultraShortPredictedPower(ultraShortMap.getOrDefault(dateTime, BigDecimal.ZERO))
                            .actualPower(powerMap.getOrDefault(dateTime, BigDecimal.ZERO))
                            .build()
            );
        }
        return XListResult.success(outputs);
    }

    /**
     * 历史风速
     * todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWindPowerStatisticsOutput }>
     */
    @XApiPost
    @XText("历史风速")
    public XSingleResult<GetWindPowerStatisticsOutput> historicalWindSpeed(XContext context, GetBaseWindTurbineViewInput input) {
        GetWindPowerStatisticsOutput output = new GetWindPowerStatisticsOutput();
        List<GetHistoricalWindSpeedOutput> speedList;
        //查电站列表
        List<BaseWindTurbineEnt> turbineList = this.getWindTurbineList(context, input.getOuId());
        if (CollUtil.isEmpty(turbineList)) {
            return XSingleResult.success(output);
        }
        List<String> collect = turbineList.stream()
                .map(BaseWindTurbineEnt::getId)
                .collect(Collectors.toList());
        //当前环境
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        ThirdWindPowerGenerationUpdateMapper mapper = context.getBean(ThirdWindPowerGenerationUpdateMapper.class);
        //确定时间范围
        DateTime beginTime, endTime;
        DateTime now = DateUtil.date();
        Map<Integer, BigDecimal> map;
        int timeType = input.getTimeType();
        if (timeType == 0) {
            beginTime = DateUtil.beginOfMonth(now);
            endTime = DateUtil.endOfMonth(now);
            map = mapper.selectList(new QueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                            .select("DAY( collect_time ) AS actual_wind_direction", "IFNULL( AVG( actual_wind_speed ), 0 ) AS actual_wind_speed")
                            .groupBy("DAY( collect_time )")
                            .lambda()
                            .between(ThirdWindPowerGenerationUpdateEnt::getCollectTime, beginTime, endTime)
                            .in(ThirdWindPowerGenerationUpdateEnt::getStationId, collect)
                            .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, oilFieldCode)
                    ).stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationUpdateEnt::getActualWindDirection, ThirdWindPowerGenerationUpdateEnt::getActualWindSpeed));
            //月度
            speedList = new ArrayList<>(now.dayOfMonth());
            for (int i = 1; i <= now.dayOfMonth(); i++) {
                speedList.add(
                        GetHistoricalWindSpeedOutput.builder()
                                .dateFlag(i)
                                .windSpeed(map.getOrDefault(i, BigDecimal.ZERO))
                                .build()
                );
            }
        } else {
            beginTime = DateUtil.beginOfYear(now);
            endTime = DateUtil.endOfYear(now);
            map = mapper.selectList(new QueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                            .select("MONTH( collect_time ) AS actual_wind_direction", "IFNULL( AVG( actual_wind_speed ), 0 ) AS actual_wind_speed")
                            .groupBy("MONTH( collect_time )")
                            .lambda()
                            .between(ThirdWindPowerGenerationUpdateEnt::getCollectTime, beginTime, endTime)
                            .in(ThirdWindPowerGenerationUpdateEnt::getStationId, collect)
                            .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, oilFieldCode)
                    ).stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationUpdateEnt::getActualWindDirection, ThirdWindPowerGenerationUpdateEnt::getActualWindSpeed));
            //年度
            speedList = new ArrayList<>(now.monthBaseOne());
            for (int i = 1; i <= now.monthBaseOne(); i++) {
                speedList.add(
                        GetHistoricalWindSpeedOutput.builder()
                                .dateFlag(i)
                                .windSpeed(map.getOrDefault(i, BigDecimal.ZERO))
                                .build()
                );
            }
        }
        //风速统计
        ThirdWindPowerGenerationUpdateEnt windSpeed = mapper.selectOne(new QueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                .select("IFNULL( MAX( actual_wind_speed ), 0 ) AS max_wind_speed",
                        "IFNULL( MIN( actual_wind_speed ), 0 ) AS min_wind_speed",
                        "IFNULL( AVG( actual_wind_speed ), 0 ) AS avg_wind_speed")
                .lambda()
                .between(ThirdWindPowerGenerationUpdateEnt::getCollectTime, beginTime, endTime)
                .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, oilFieldCode)
                .in(ThirdWindPowerGenerationUpdateEnt::getStationName, collect)
        );
        //平均风速
        output.setMaxWindSpeed(windSpeed.getMaxWindSpeed());
        output.setMinWindSpeed(windSpeed.getMinWindSpeed());
        output.setAvgWindSpeed(windSpeed.getAvgWindSpeed());
        //风速列表
        output.setSpeedList(speedList);
        return XSingleResult.success(output);
    }

    /**
     * 历史风向
     *  todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetHistoricalWindDirectionOutput }>
     */
    @XApiPost
    @XText("历史风向")
    public XSingleResult<GetHistoricalWindDirectionOutput> historicalWindDirection(XContext context, GetBaseWindTurbineViewInput input) {
        GetHistoricalWindDirectionOutput output = new GetHistoricalWindDirectionOutput();
        //查电站列表
        List<BaseWindTurbineEnt> turbineList = this.getWindTurbineList(context, input.getOuId());
        if (CollUtil.isEmpty(turbineList)) {
            return XSingleResult.success(output);
        }
        //确定时间范围
        DateTime beginTime, endTime;
        DateTime now = DateUtil.date();
        int timeType = input.getTimeType();
        if (timeType == 0) {
            beginTime = DateUtil.beginOfMonth(now);
            endTime = DateUtil.endOfMonth(now);
        } else {
            beginTime = DateUtil.beginOfYear(now);
            endTime = DateUtil.endOfYear(now);
        }
        //分组查询风向
        ThirdWindPowerGenerationMapper futureMapper = context.getBean(ThirdWindPowerGenerationMapper.class);
        List<ThirdWindPowerGenerationEnt> list = futureMapper.selectList(new QueryWrapper<ThirdWindPowerGenerationEnt>()
                .select("actual_wind_direction", "COUNT( 1 ) AS actual_wind_speed")
                .lambda()
                .between(ThirdWindPowerGenerationEnt::getCollectTime, beginTime, endTime)
                .in(ThirdWindPowerGenerationEnt::getStationId, turbineList.stream()
                        .map(BaseWindTurbineEnt::getId)
                        .collect(Collectors.toList()))
                .eq(ThirdWindPowerGenerationEnt::getSystemSource, ServiceUtil.getOilFieldCode(context))
                .groupBy(ThirdWindPowerGenerationEnt::getActualWindDirection)
        );
        if (CollUtil.isEmpty(list)) {
            return XSingleResult.success(output);
        }
        //取总数
        BigDecimal total = BigDecimal.ZERO;
        for (ThirdWindPowerGenerationEnt future : list) {
            total = total.add(future.getActualWindSpeed());
            switch (future.getActualWindDirection()) {
                case 0:
                    output.setNorth(future.getActualWindSpeed());
                    break;
                case 45:
                    output.setNortheast(future.getActualWindSpeed());
                    break;
                case 90:
                    output.setEast(future.getActualWindSpeed());
                    break;
                case 135:
                    output.setSoutheast(future.getActualWindSpeed());
                    break;
                case 180:
                    output.setSouth(future.getActualWindSpeed());
                    break;
                case 225:
                    output.setSouthwest(future.getActualWindSpeed());
                    break;
                case 270:
                    output.setWest(future.getActualWindSpeed());
                    break;
                case 315:
                    output.setNorthwest(future.getActualWindSpeed());
                    break;
            }
        }
        //计算率
        output.setNorth(BaseUtils.getRate(output.getNorth(), total));
        output.setNortheast(BaseUtils.getRate(output.getNortheast(), total));
        output.setEast(BaseUtils.getRate(output.getEast(), total));
        output.setSoutheast(BaseUtils.getRate(output.getSoutheast(), total));
        output.setSouth(BaseUtils.getRate(output.getSouth(), total));
        output.setSouthwest(BaseUtils.getRate(output.getSouthwest(), total));
        output.setWest(BaseUtils.getRate(output.getWest(), total));
        output.setNorthwest(BaseUtils.getRate(output.getNorthwest(), total));
        return XSingleResult.success(output);
    }

    /**
     * 风电站运行状态
     * todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWindTurbineRunStatusOutput }>
     */
    @XApiPost
    @XText("风电站运行状态")
    public XListResult<GetWindTurbineRunStatusOutput> windTurbineRunStatusList(XContext context, GetBaseWindTurbineViewInput input) {
        if (CharSequenceUtil.isNotBlank(input.getRunState()) && !CharSequenceUtil.equals(input.getRunState(), "启动")) {
            return XListResult.success(Collections.emptyList());
        }
        List<BaseWindTurbineEnt> turbineList = this.getWindTurbineList(context, input.getOuId());
        if (CollUtil.isEmpty(turbineList)) {
            return XListResult.success(Collections.emptyList());
        }
        List<GetWindTurbineRunStatusOutput> outputs = new ArrayList<>(turbineList.size());
        //实际数据
        ThirdWindPowerGenerationUpdateMapper generationMapper = context.getBean(ThirdWindPowerGenerationUpdateMapper.class);
        List<ThirdWindPowerGenerationUpdateEnt> list = generationMapper.selectList(new LambdaQueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                .in(ThirdWindPowerGenerationUpdateEnt::getStationId, turbineList.stream()
                        .map(BaseModel::getId)
                        .collect(Collectors.toList()))
                .ge(ThirdWindPowerGenerationUpdateEnt::getCollectTime, DateUtil.beginOfDay(DateUtil.date()))
                .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, ServiceUtil.getOilFieldCode(context))
        );
        Map<String, ThirdWindPowerGenerationUpdateEnt> collect;
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationUpdateEnt::getStationId, Function.identity()));
        }
        //封装数据
        GetWindTurbineRunStatusOutput output;
        ThirdWindPowerGenerationUpdateEnt ent;
        for (BaseWindTurbineEnt turbine : turbineList) {
            output = new GetWindTurbineRunStatusOutput();
            output.setStationId(turbine.getId());
            output.setStationName(turbine.getStationName());
            output.setRunState("启动");
            if (collect.containsKey(turbine.getId())) {
                ent = collect.get(turbine.getId());
                output.setActivePower(ent.getActualPower());
                output.setActualWindSpeed(ent.getActualWindSpeed());
            } else {
                output.setActivePower(BigDecimal.ZERO);
                output.setActualWindSpeed(BigDecimal.ZERO);
            }
            outputs.add(output);
        }
        return XListResult.success(outputs);
    }

    /**
     * 风机运行情况
     * todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWeatherBureauDataOutput }>
     */
    @XApiPost
    @XText("风机运行情况")
    public XSingleResult<GetBaseWindTurbineViewOutput> workCondition(XContext context, GetWindPredictionFutureInput input) {
        GetBaseWindTurbineViewOutput output = new GetBaseWindTurbineViewOutput();
        //风机详情
        BaseWindTurbineViewMapper mapper = context.getBean(BaseWindTurbineViewMapper.class);
        BaseWindTurbineView turbineView = mapper.selectOne(input.getStationId());
        if (ObjectUtil.isNull(turbineView)) {
            return XSingleResult.success(output);
        }
        //实时天气
        DateTime endTime = DateUtil.date();
        DateTime beginTime = DateUtil.offsetHour(endTime, -1);
        WindPredictionFutureMapper futureMapper = context.getBean(WindPredictionFutureMapper.class);
        WindPredictionFutureEnt futureEnt = futureMapper.selectOne(new LambdaQueryWrapper<WindPredictionFutureEnt>()
                .eq(WindPredictionFutureEnt::getStationId, input.getStationId())
                .between(WindPredictionFutureEnt::getDataTime, beginTime, endTime)
                .orderByDesc(WindPredictionFutureEnt::getDataTime)
                .last(BusinessConstant.LAST_LIMIT)
        );
        if (ObjectUtil.isNotNull(futureEnt)) {
            output.setWindDirection(WindDirection.findByCode(futureEnt.getWindDirection()));
            output.setWindSpeed(futureEnt.getWindSpeed());
            output.setAirTemperature(futureEnt.getAirTemperature());
            output.setHumidity(futureEnt.getHumidity());
            output.setPressure(futureEnt.getPressure());
        }
        output.setStationName(turbineView.getStationName());
        output.setTotalPower(turbineView.getTotalPower());
        output.setDiameter(turbineView.getDiameter());
        output.setVaneLength(turbineView.getVaneLength());
        output.setHubHeight(turbineView.getHubHeight());
        output.setAreaName(turbineView.getCityName());
        //环境
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //模拟数据
        ThirdWindPowerGenerationUpdateMapper bean = context.getBean(ThirdWindPowerGenerationUpdateMapper.class);
        ThirdWindPowerGenerationUpdateEnt selectOne = bean.selectOne(new LambdaQueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                .eq(ThirdWindPowerGenerationUpdateEnt::getStationId, input.getStationId())
                .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, oilFieldCode)
                .orderByDesc(ThirdWindPowerGenerationUpdateEnt::getCollectTime)
                .last(BusinessConstant.LAST_LIMIT)
        );
        //todo 型号暂时不知道从哪来
        output.setModelNumber(BusinessConstant.DEFAULT_VALUE);
        output.setDailyGeneration(selectOne.getActualGeneration());
        output.setActivePower(selectOne.getActualPower());
        output.setActualWindSpeed(selectOne.getActualWindSpeed());
        //总计数据
        selectOne = bean.selectOne(new QueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                .select("SUM( actual_generation ) AS actual_generation")
                .lambda()
                .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, oilFieldCode)
                .eq(ThirdWindPowerGenerationUpdateEnt::getStationId, input.getStationId())
        );
        output.setGrossGeneration(selectOne.getActualGeneration());
        output.setRunTime(BigDecimal.ZERO);
        output.setRunState("启动");
        return XSingleResult.success(output);
    }

    /**
     * 发电功率列表
     *  todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWindActivePowerOutput }>
     */
    @XApiPost
    @XText("发电功率列表")
    public XListResult<GetWindActivePowerOutput> activePowerList(XContext context, GetWindPredictionFutureInput input) {
        BaseWindTurbineViewMapper mapper = context.getBean(BaseWindTurbineViewMapper.class);
        BaseWindTurbineView turbineView = mapper.selectOne(input.getStationId());
        if (ObjectUtil.isNull(turbineView)) {
            return XListResult.success(Collections.emptyList());
        }
        //计算时间
        DateTime now = DateUtil.date();
        DateTime beginTime = DateUtil.beginOfDay(now);
        //封装
        DateTime endTime = DateUtil.endOfDay(beginTime);
        //查数据
        ThirdWindPowerGenerationMapper generationMapper = context.getBean(ThirdWindPowerGenerationMapper.class);
        List<ThirdWindPowerGenerationEnt> list = generationMapper.selectList(new QueryWrapper<ThirdWindPowerGenerationEnt>()
                .select("collect_time", "IFNULL( SUM( actual_power ), 0 ) AS actual_power")
                .lambda()
                .between(ThirdWindPowerGenerationEnt::getCollectTime, beginTime, endTime)
                .eq(ThirdWindPowerGenerationEnt::getStationId, input.getStationId())
                .eq(ThirdWindPowerGenerationEnt::getSystemSource, ServiceUtil.getOilFieldCode(context))
                .groupBy(ThirdWindPowerGenerationEnt::getCollectTime)
        );
        List<GetWindActivePowerOutput> outputs = new ArrayList<>();
        if (CollUtil.isNotEmpty(list)) {
            Map<Date, BigDecimal> map = list.stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationEnt::getCollectTime, ThirdWindPowerGenerationEnt::getActualPower));
            for (DateTime dateTime : DateUtil.rangeToList(beginTime, endTime, DateField.MINUTE, 15)) {
                outputs.add(
                        GetWindActivePowerOutput.builder()
                                .dataTime(dateTime)
                                .activePower(map.getOrDefault(dateTime, BigDecimal.ZERO))
                                .build()
                );
            }
        }
        return XListResult.success(outputs);
    }

    /**
     * 发电趋势列表
     * 油厂平均：该月该油厂下的风机发电总和/该油厂下的风机数
     * todo 模拟数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWindPowerTrendOutput }>
     */
    @XApiPost
    @XText("发电趋势列表")
    public XListResult<GetWindPowerTrendOutput> powerTrendList(XContext context, GetWindPredictionFutureInput input) {
        BaseWindTurbineViewMapper mapper = context.getBean(BaseWindTurbineViewMapper.class);
        BaseWindTurbineView turbineView = mapper.selectOne(input.getStationId());
        if (ObjectUtil.isNull(turbineView)) {
            return XListResult.success(Collections.emptyList());
        }
        DateTime beginOfYear = DateUtil.beginOfYear(DateUtil.date());
        DateTime endOfYear = DateUtil.endOfYear(beginOfYear);
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //查本年发电
        ThirdWindPowerGenerationUpdateMapper generationMapper = context.getBean(ThirdWindPowerGenerationUpdateMapper.class);
        List<ThirdWindPowerGenerationUpdateEnt> list = generationMapper.selectList(new QueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                .select("IFNULL( SUM( actual_generation ), 0 ) AS actual_generation", "MONTH ( collect_time ) AS actual_wind_direction")
                .groupBy("MONTH ( collect_time )")
                .lambda()
                .eq(ThirdWindPowerGenerationUpdateEnt::getStationId, input.getStationId())
                .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, oilFieldCode)
                .between(ThirdWindPowerGenerationUpdateEnt::getCollectTime, beginOfYear, endOfYear)
        );
        Map<Integer, BigDecimal> nowMap;
        if (CollUtil.isNotEmpty(list)) {
            nowMap = list.stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationUpdateEnt::getActualWindDirection, ThirdWindPowerGenerationUpdateEnt::getActualGeneration));
        } else {
            nowMap = Collections.emptyMap();
        }
        //油厂平均
        Map<Integer, BigDecimal> ouMap;
        List<BaseWindTurbineEnt> turbineList = this.getWindTurbineList(context, turbineView.getOuId());
        if (CollUtil.isEmpty(turbineList)) {
            ouMap = Collections.emptyMap();
        } else {
            ThirdWindPowerGenerationUpdateViewMapper viewMapper = context.getBean(ThirdWindPowerGenerationUpdateViewMapper.class);
            List<ThirdWindPowerGenerationUpdateView> viewList = viewMapper.stationMonthAvgPowerGeneration(turbineList.stream()
                    .map(BaseModel::getId)
                    .collect(Collectors.toList()), beginOfYear, endOfYear);
            if (CollUtil.isEmpty(viewList)) {
                ouMap = Collections.emptyMap();
            } else {
                ouMap = viewList.stream()
                        .collect(Collectors.toMap(ThirdWindPowerGenerationUpdateView::getActualWindDirection, ThirdWindPowerGenerationUpdateView::getActualGeneration));
            }
        }
        //查去年发电
        beginOfYear = DateUtil.offset(beginOfYear, DateField.YEAR, -1);
        endOfYear = DateUtil.offset(endOfYear, DateField.YEAR, -1);
        list = generationMapper.selectList(new QueryWrapper<ThirdWindPowerGenerationUpdateEnt>()
                .select("IFNULL( SUM( actual_generation ), 0 ) AS actual_generation", "MONTH ( collect_time ) AS actual_wind_direction")
                .groupBy("MONTH ( collect_time )")
                .lambda()
                .eq(ThirdWindPowerGenerationUpdateEnt::getStationId, input.getStationId())
                .eq(ThirdWindPowerGenerationUpdateEnt::getSystemSource, oilFieldCode)
                .between(ThirdWindPowerGenerationUpdateEnt::getCollectTime, beginOfYear, endOfYear)
        );
        Map<Integer, BigDecimal> lastMap;
        if (CollUtil.isNotEmpty(list)) {
            lastMap = list.stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationUpdateEnt::getActualWindDirection, ThirdWindPowerGenerationUpdateEnt::getActualGeneration));
        } else {
            lastMap = Collections.emptyMap();
        }
        //匹配数据
        List<GetWindPowerTrendOutput> outputs = new ArrayList<>(12);
        for (int i = 1; i <= 12; i++) {
            outputs.add(
                    GetWindPowerTrendOutput.builder()
                            .month(i)
                            .activePower(nowMap.getOrDefault(i, BigDecimal.ZERO))
                            .lastActivePower(lastMap.getOrDefault(i, BigDecimal.ZERO))
                            .avgActivePower(ouMap.getOrDefault(i, BigDecimal.ZERO))
                            .build()
            );
        }
        return XListResult.success(outputs);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 获得预测功率输出
     *
     * @param context      上下文
     * @param inputTimeStr 输入时间str
     * @param stationId    站点id
     * @param offsetDay    补偿日
     * @return {@link List }<{@link GetWindPredictionFutureOutput }>
     */
    private List<GetWindPredictionFutureOutput> getPredictedPowerOutput(XContext context, String inputTimeStr, String stationId, int offsetDay) {
        //判断是否需要启用演示配置
        DateTime date = DateUtil.date();
        DateTime inputTime;
        if (CharSequenceUtil.isBlank(inputTimeStr)) {
            inputTime = DateUtil.beginOfDay(date);
        } else {
            inputTime = DateUtil.parse(inputTimeStr, BusinessConstant.DATE_FORMAT_DAY);
        }
        //时间出力截至时间
        DateTime endTime = DateUtil.offsetDay(inputTime, offsetDay);
        return getPowerOutput(context, inputTime, endTime, 0, stationId);
    }

    /**
     * 获得预测功率输出
     *
     * @param context    上下文
     * @param beginTime  开始时间
     * @param endTime    结束时间
     * @param betweenDay 日间
     * @param stationId  站点id
     * @return {@link List }<{@link GetWindPredictionFutureOutput }>
     */
    private List<GetWindPredictionFutureOutput> getPowerOutput(XContext context, DateTime beginTime, DateTime endTime, int betweenDay, String stationId) {
        //查预测功率
        Map<Date, WindPredictionFutureEnt> predictedMap = this.getPredictedMap(context, stationId, beginTime, endTime);
        //查实际功率
        Map<Date, ThirdWindPowerGenerationEnt> powerMap = this.getActivePower15MinuteMap(context, stationId, beginTime, endTime);
        //取时间范围
        List<DateTime> rangeToList = DateUtil.rangeToList(beginTime, endTime, DateField.MINUTE, 15);
        GetWindPredictionFutureOutput output;
        List<GetWindPredictionFutureOutput> outputs = new ArrayList<>(rangeToList.size());
        Date dataDate;
        WindPredictionFutureEnt windPredictionFutureEnt;
        ThirdWindPowerGenerationEnt generationEnt;
        for (DateTime dateTime : rangeToList) {
            output = new GetWindPredictionFutureOutput();
            //时间偏移
            if (betweenDay != 0) {
                dataDate = DateUtil.offsetDay(dateTime, betweenDay);
            } else {
                dataDate = dateTime;
            }
            output.setDataTime(dataDate);
            //匹配预测发电量
            if (predictedMap.containsKey(dataDate)) {
                windPredictionFutureEnt = predictedMap.get(dataDate);
                output.setPredictedPower(windPredictionFutureEnt.getPredictedPower());
                output.setWindSpeed(windPredictionFutureEnt.getWindSpeed());
            }
            //匹配实际发电量
            if (powerMap.containsKey(dataDate)) {
                generationEnt = powerMap.get(dataDate);
                output.setActualWindSpeed(generationEnt.getActualWindSpeed());
                output.setActualPower(generationEnt.getActualPower());
            }
            outputs.add(output);
        }
        return outputs;
    }

    /**
     * 获取预测数据
     *
     * @param context   上下文
     * @param stationId 站点id
     * @param beginTime 开始时间
     * @param endTime   结束时间
     * @return {@link Map }<{@link Date }, {@link WindPredictionFutureEnt }>
     */
    private Map<Date, WindPredictionFutureEnt> getPredictedMap(XContext context, String stationId, DateTime beginTime, DateTime endTime) {
        WindPredictionFutureMapper mapper = context.getBean(WindPredictionFutureMapper.class);
        List<WindPredictionFutureEnt> predictedList = mapper.selectList(new QueryWrapper<WindPredictionFutureEnt>()
                .select("data_time", "IFNULL( MAX( predicted_power ), 0 ) AS predicted_power",
                        "IFNULL( MAX( actual_wind_speed ), 0 ) AS actual_wind_speed", "IFNULL( MAX( wind_speed ), 0 ) AS wind_speed")
                .lambda()
                .eq(WindPredictionFutureEnt::getStationId, stationId)
                .between(WindPredictionFutureEnt::getDataTime, beginTime, endTime)
                .groupBy(WindPredictionFutureEnt::getDataTime, WindPredictionFutureEnt::getActualWindSpeed,
                        WindPredictionFutureEnt::getWindSpeed)
                .orderByAsc(WindPredictionFutureEnt::getDataTime)
        );
        Map<Date, WindPredictionFutureEnt> predictedMap;
        if (CollUtil.isNotEmpty(predictedList)) {
            predictedMap = predictedList.stream()
                    .collect(Collectors.toMap(WindPredictionFutureEnt::getDataTime, Function.identity()));
        } else {
            predictedMap = Collections.emptyMap();
        }
        return predictedMap;
    }

    /**
     * 获取预测数据
     *
     * @param context    上下文
     * @param stationIds 站点ID
     * @param beginTime  开始时间
     * @param endTime    结束时间
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    private Map<Date, BigDecimal> getPredictedMap(XContext context, List<String> stationIds, Date beginTime, Date endTime) {
        WindPredictionFutureMapper mapper = context.getBean(WindPredictionFutureMapper.class);
        List<WindPredictionFutureEnt> predictedList = mapper.selectList(new QueryWrapper<WindPredictionFutureEnt>()
                .select("data_time", "IFNULL( SUM( predicted_power ), 0 ) AS predicted_power")
                .lambda()
                .in(WindPredictionFutureEnt::getStationId, stationIds)
                .between(WindPredictionFutureEnt::getDataTime, beginTime, endTime)
                .groupBy(WindPredictionFutureEnt::getDataTime)
                .orderByAsc(WindPredictionFutureEnt::getDataTime)
        );
        Map<Date, BigDecimal> predictedMap;
        if (CollUtil.isNotEmpty(predictedList)) {
            predictedMap = predictedList.stream()
                    .collect(Collectors.toMap(WindPredictionFutureEnt::getDataTime, WindPredictionFutureEnt::getPredictedPower));
        } else {
            predictedMap = Collections.emptyMap();
        }
        return predictedMap;
    }

    /**
     * 获取风力电站列表
     *
     * @param context 上下文
     * @param ouId    你id
     * @return {@link List }<{@link BaseWindTurbineEnt }>
     */
    private List<BaseWindTurbineEnt> getWindTurbineList(XContext context, String ouId) {
        //找组织
        LambdaQueryWrapper<BaseWindTurbineEnt> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(BaseModel::getIsDeleted, BusinessConstant.ONE);
        if (CharSequenceUtil.isBlank(ouId)) {
            ouId = context.getSession(PpsUserSession.class).getOuId();
        }
        List<GetSysOrganizationViewOutput> ouList = ServiceUtil.getOrgIdsByPath(context, GetAllOuListByOuIdInput.builder()
                .ouId(ouId)
                .build());
        queryWrapper.in(BaseWindTurbineEnt::getOuId, ouList.stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList())
        );
        //查电站
        BaseWindTurbineMapper mapper = context.getBean(BaseWindTurbineMapper.class);
        return mapper.selectList(queryWrapper);
    }

    /**
     * 获取真实发电功率
     * todo 模拟数据
     *
     * @param context   上下文
     * @param stationId 站点id
     * @param beginTime 开始时间
     * @param endTime   结束时间
     * @return {@link Map }<{@link Date }, {@link ThirdWindPowerGenerationEnt }>
     */
    private Map<Date, ThirdWindPowerGenerationEnt> getActivePower15MinuteMap(XContext context, String stationId, DateTime beginTime, DateTime endTime) {
        ThirdWindPowerGenerationMapper generationMapper = context.getBean(ThirdWindPowerGenerationMapper.class);
        List<ThirdWindPowerGenerationEnt> list = generationMapper.selectList(new QueryWrapper<ThirdWindPowerGenerationEnt>()
                .select("collect_time", "IFNULL( MAX( actual_wind_speed ), 0 ) AS actual_wind_speed", "IFNULL( MAX( actual_power ), 0 ) AS actual_power")
                .groupBy("collect_time")
                .lambda()
                .eq(ThirdWindPowerGenerationEnt::getStationId, stationId)
                .eq(ThirdWindPowerGenerationEnt::getSystemSource, ServiceUtil.getOilFieldCode(context))
                .between(ThirdWindPowerGenerationEnt::getCollectTime, beginTime, endTime)
        );
        Map<Date, ThirdWindPowerGenerationEnt> collect;
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream().collect(Collectors.toMap(ThirdWindPowerGenerationEnt::getCollectTime, Function.identity()));
        }
        return collect;
    }
}